<?php

namespace App\Services\marketing;

use App\Exception\custom\MarketingException;
use App\Models\marketing\mysql\ColonelDistributorColonel;
use App\Models\marketing\mysql\ColonelDistributorInviteOrder;
use App\Models\marketing\mysql\ColonelDistributorInviteOrderNum;
use App\Models\marketing\mysql\ColonelDistributorPayInfo;

class PindanActivityInviteOrderService
{
    /**
     * 团长分销下单逻辑
     * @param array $params
     * @return bool
     * @throws MarketingException
     */
    public static function placeOrder($params = [])
    {
        $num = empty($params["num"]) ? 1 : $params["num"];
        $inviteOrderData = [
            "colonel_user_id" => $params["colonel_user_id"],
            "user_id" => $params["user_id"],
            "order_id" => $params["order_id"],
            "refund_time" => null,
        ];

        $inviteOrder = ColonelDistributorInviteOrder::selectMaster("*", $inviteOrderData, []);
        if (!empty($inviteOrder)) {
            return true;
        }

        $inviteOrderNumData = [
            "colonel_user_id" => $params["colonel_user_id"],
            "date" => date("Y-m-d"),
        ];
        $inviteOrderNum = ColonelDistributorInviteOrderNum::selectMaster("*", $inviteOrderNumData, []);

        ColonelDistributorInviteOrder::beginTransaction();
        $addInviteOrderRes = ColonelDistributorInviteOrder::save($inviteOrderData);

        if (!empty($inviteOrderNum)) {
            $addInviteOrderNumRes = ColonelDistributorInviteOrderNum::save(["num[+]" => $num], ["invite_order_num_id" => $inviteOrderNum[0]["invite_order_num_id"]]);
        } else {
            $inviteOrderNumData["num"] = $num;
            $addInviteOrderNumRes = ColonelDistributorInviteOrderNum::save($inviteOrderNumData);
        }

        if (empty($addInviteOrderRes) || empty($addInviteOrderNumRes) || !ColonelDistributorInviteOrder::commit()) {
            ColonelDistributorInviteOrder::rollback();
            throw new MarketingException(["cus" => MarketingException::COMMIT_ERROR]);
        }
        return true;
    }

    /**
     * 团长分销退单逻辑
     * @param array $params
     * @return bool
     * @throws MarketingException
     */
    public static function refundOrder($params = [])
    {
        $orderId = $params["order_id"];

        $inviteOrder = ColonelDistributorInviteOrder::selectMaster("*", ["order_id" => $orderId], []);
        if (!empty($inviteOrder) && $inviteOrder[0]["is_refund"] == ColonelDistributorInviteOrder::IS_REFUND_NO) {
            $inviteOrderData = [
                "colonel_user_id" => $inviteOrder[0]["colonel_user_id"],
                "user_id" => $inviteOrder[0]["user_id"],
                "order_id" => $orderId,
            ];

            $inviteOrderNumData = [
                "colonel_user_id" => $inviteOrder[0]["colonel_user_id"],
                "date" => date("Y-m-d"),
            ];

            ColonelDistributorInviteOrder::beginTransaction();
            $addInviteOrderRes = ColonelDistributorInviteOrder::save([
                "is_refund" => ColonelDistributorInviteOrder::IS_REFUND_YES,
                "refund_time" => date("y-m-d H:i:s")
            ], $inviteOrderData);

            $addInviteOrderNumRes = ColonelDistributorInviteOrderNum::save(["num[-]" => 1], $inviteOrderNumData);

            if (empty($addInviteOrderRes) || empty($addInviteOrderNumRes) || !ColonelDistributorInviteOrder::commit()) {
                ColonelDistributorInviteOrder::rollback();
                throw new MarketingException(["cus" => MarketingException::COMMIT_ERROR]);
            }
        }
        return true;
    }

    /**
     * 团长分销，昨日收入，总收入
     * @param array $params
     * @return array
     */
    public static function incomeStatistics($params = [])
    {
        $userIds = $params["user_ids"];
        $yesterday = date("Y-m-d", strtotime("-1 day"));

        $yesterdayRewards = ColonelDistributorPayInfo::select(["colonel_user_id", "reward"], ["date" => $yesterday, "colonel_user_id" => $userIds], []);
        $yesterdayData = [];
        if (!empty($yesterdayRewards)) {
            foreach ($yesterdayRewards as $item) {
                $yesterdayData[$item["colonel_user_id"]][] = $item["reward"];
            }
        }

        $allRewards = ColonelDistributorPayInfo::select(["colonel_user_id", "reward"], ["colonel_user_id" => $userIds], []);
        $allData = [];
        if (!empty($allRewards)) {
            foreach ($allRewards as $item) {
                $allData[$item["colonel_user_id"]][] = $item["reward"];
            }
        }

        $data = [];
        foreach ($userIds as $userId) {
            $data[$userId]["yesterday_reward"] = 0;
            $data[$userId]["all_reward"] = 0;
            if (!empty($yesterdayData[$userId])) {
                $data[$userId]["yesterday_reward"] = sprintf("%.2f", array_sum($yesterdayData[$userId]) / 100);
            }
            if (!empty($allData[$userId])) {
                $data[$userId]["all_reward"] = sprintf("%.2f", array_sum($allData[$userId]) / 100);
            }
        }

        return $data;
    }

    /**
     * 团长分销，昨日转化单数，总转化单数
     * @param array $params
     * @return array
     */
    public static function inviteOrderNumberStatistics($params = [])
    {
        $userIds = $params["user_ids"];
        $yesterday = date("Y-m-d", strtotime("-1 day"));

        $yesterdayNums = ColonelDistributorInviteOrderNum::select(["colonel_user_id", "num"],
            ["date" => $yesterday, "colonel_user_id" => $userIds], []);
        $yesterdayData = array_column($yesterdayNums, null, "colonel_user_id");

        $allNums = ColonelDistributorInviteOrderNum::select(["colonel_user_id", "num"], ["colonel_user_id" => $userIds], []);
        $allData = array_column($allNums, null, "colonel_user_id");

        $data = [];
        foreach ($userIds as $userId) {
            $data[$userId]["yesterday_order_num"] = 0;
            $data[$userId]["all_order_num"] = 0;
            if (!empty($yesterdayData[$userId])) {
                $data[$userId]["yesterday_order_num"] = $yesterdayData[$userId]["num"];
            }
            if (!empty($allData[$userId])) {
                $data[$userId]["all_order_num"] = $allData[$userId]["num"];
            }
        }
        return $data;
    }
}